# Stub file that should contain all public methods of AXUtilities
#
# Copyright 2023 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=missing-function-docstring
# pylint: disable=unused-argument
# pylint: disable=too-many-lines
# pylint: disable=missing-class-docstring
# pylint: disable=wrong-import-position
# pylint: disable=too-many-public-methods

"""
Stub file that contains all public methods of AXUtilities, including
those which are added dynamically at runtime. This is needed for type
checkers, linters, and IDEs.
"""

__id__        = "$Id$"
__version__   = "$Revision$"
__date__      = "$Date$"
__copyright__ = "Copyright (c) 2023 Igalia, S.L."
__license__   = "LGPL"

from typing import Callable

import gi
gi.require_version("Atspi", "2.0")
from gi.repository import Atspi

from .ax_utilities_event import TextEventReason

class AXUtilities:

    # From ax_utilities.py
    @staticmethod
    def clear_all_cache_now(obj: Atspi.Accessible | None = None, reason: str = "") -> None: ...

    @staticmethod
    def can_be_active_window(window: Atspi.Accessible) -> bool: ...

    @staticmethod
    def find_active_window() -> Atspi.Accessible | None: ...

    @staticmethod
    def is_unfocused_alert_or_dialog(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def get_unfocused_alerts_and_dialogs(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_all_widgets(
        obj: Atspi.Accessible,
        must_be_showing_and_visible: bool = True,
        exclude_push_button: bool = False
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_default_button(obj: Atspi.Accessible) -> Atspi.Accessible | None: ...

    @staticmethod
    def get_focused_object(obj: Atspi.Accessible) -> Atspi.Accessible | None: ...

    @staticmethod
    def get_info_bar(obj: Atspi.Accessible) -> Atspi.Accessible | None: ...

    @staticmethod
    def get_status_bar(obj: Atspi.Accessible) -> Atspi.Accessible | None: ...

    @staticmethod
    def is_layout_only(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_message_dialog(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_redundant_object(obj1: Atspi.Accessible, obj2: Atspi.Accessible) -> bool: ...

    @staticmethod
    def get_set_members(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_set_size(obj: Atspi.Accessible) -> int: ...

    @staticmethod
    def get_set_size_is_unknown(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def get_position_in_set(obj: Atspi.Accessible) -> int: ...

    @staticmethod
    def has_explicit_name(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def has_visible_caption(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def get_displayed_label(obj: Atspi.Accessible) -> str: ...

    @staticmethod
    def get_displayed_description(obj: Atspi.Accessible) -> str: ...

    @staticmethod
    def get_heading_level(obj: Atspi.Accessible) -> int: ...

    @staticmethod
    def get_nesting_level(obj: Atspi.Accessible) -> int: ...

    @staticmethod
    def get_next_object(obj: Atspi.Accessible) -> Atspi.Accessible | None: ...

    @staticmethod
    def get_previous_object(obj: Atspi.Accessible) -> Atspi.Accessible | None: ...

    @staticmethod
    def is_on_screen(obj: Atspi.Accessible, bounding_box: Atspi.Rect | None = None) -> bool: ...

    @staticmethod
    def treat_as_leaf_node(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def get_on_screen_objects(
        root: Atspi.Accessible,
        bounding_box: Atspi.Rect | None = None
    ) -> list: ...

    # From ax_utilities_application.py
    @staticmethod
    def application_as_string(obj: Atspi.Accessible) -> str: ...

    @staticmethod
    def get_all_applications(
        must_have_window: bool = False,
        exclude_unresponsive: bool = False,
        is_debug: bool = False
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_application_with_pid(pid: int) -> Atspi.Accessible | None: ...

    @staticmethod
    def get_application(obj: Atspi.Accessible) -> Atspi.Accessible | None: ...

    @staticmethod
    def get_application_toolkit_name(obj: Atspi.Accessible) -> str: ...

    @staticmethod
    def get_application_toolkit_version(obj: Atspi.Accessible) -> str: ...

    @staticmethod
    def get_desktop() -> Atspi.Accessible: ...

    @staticmethod
    def get_process_id(obj: Atspi.Accessible) -> int: ...

    @staticmethod
    def is_application_in_desktop(app: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_application_unresponsive(app: Atspi.Accessible) -> bool: ...

    # From ax_utilities_event.py
    @staticmethod
    def save_object_info_for_events(obj: Atspi.Accessible) -> None: ...

    @staticmethod
    def get_text_event_reason(event: Atspi.Event) -> TextEventReason: ...

    @staticmethod
    def is_presentable_active_descendant_change(event: Atspi.Event) -> bool: ...

    @staticmethod
    def is_presentable_checked_change(event: Atspi.Event) -> bool: ...

    @staticmethod
    def is_presentable_description_change(event: Atspi.Event) -> bool: ...

    @staticmethod
    def is_presentable_expanded_change(event: Atspi.Event) -> bool: ...

    @staticmethod
    def is_presentable_indeterminate_change(event: Atspi.Event) -> bool: ...

    @staticmethod
    def is_presentable_invalid_entry_change(event: Atspi.Event) -> bool: ...

    @staticmethod
    def is_presentable_name_change(event: Atspi.Event) -> bool: ...

    @staticmethod
    def is_presentable_pressed_change(event: Atspi.Event) -> bool: ...

    @staticmethod
    def is_presentable_selected_change(event: Atspi.Event) -> bool: ...

    @staticmethod
    def is_presentable_text_attributes_change(event: Atspi.Event) -> bool: ...

    @staticmethod
    def is_presentable_text_deletion(event: Atspi.Event) -> bool: ...

    @staticmethod
    def is_presentable_text_insertion(event: Atspi.Event) -> bool: ...

    # From ax_utilities_role.py
    @staticmethod
    def _get_display_style(obj: Atspi.Accessible) -> str: ...

    @staticmethod
    def _get_tag(obj: Atspi.Accessible) -> str | None: ...

    @staticmethod
    def _get_xml_roles(obj: Atspi.Accessible) -> list[str]: ...

    @staticmethod
    def children_are_presentational(
        obj: Atspi.Accessible,
        role: Atspi.Role | None = None
    ) -> bool: ...

    @staticmethod
    def get_dialog_roles(include_alert_as_dialog: bool = True) -> list[Atspi.Role]: ...

    @staticmethod
    def get_document_roles() -> list[Atspi.Role]: ...

    @staticmethod
    def get_form_field_roles() -> list[Atspi.Role]: ...

    @staticmethod
    def get_large_container_roles() -> list[Atspi.Role]: ...

    @staticmethod
    def get_layout_only_roles() -> list[Atspi.Role]: ...

    @staticmethod
    def get_menu_item_roles() -> list[Atspi.Role]: ...

    @staticmethod
    def get_menu_related_roles() -> list[Atspi.Role]: ...

    @staticmethod
    def get_roles_to_exclude_from_clickables_list() -> list[Atspi.Role]: ...

    @staticmethod
    def get_set_container_roles() -> list[Atspi.Role]: ...

    @staticmethod
    def get_table_cell_roles(include_headers: bool = True) -> list[Atspi.Role]: ...

    @staticmethod
    def get_table_header_roles() -> list[Atspi.Role]: ...

    @staticmethod
    def get_table_related_roles(include_caption: bool = False) -> list[Atspi.Role]: ...

    @staticmethod
    def get_text_ui_roles() -> list[Atspi.Role]: ...

    @staticmethod
    def get_tree_related_roles() -> list[Atspi.Role]: ...

    @staticmethod
    def get_widget_roles() -> list[Atspi.Role]: ...

    @staticmethod
    def get_localized_role_name(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> str: ...

    @staticmethod
    def has_role_from_aria(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def have_same_role(obj1: Atspi.Accessible, obj2: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_accelerator_label(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_alert(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_aria_alert(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_animation(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_application(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_arrow(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_article(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_audio(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_autocomplete(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_block_quote(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_button(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_button_with_popup(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_calendar(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_canvas(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_caption(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_chart(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_check_box(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_check_menu_item(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_code(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_color_chooser(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_column_header(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_combo_box(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_comment(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_content_deletion(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_content_insertion(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_default_button(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_date_editor(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_definition(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_description_list(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_description_term(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_description_value(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_desktop_frame(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_desktop_icon(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dial(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dialog(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dialog_or_alert(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dialog_or_window(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_directory_pane(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_docked_frame(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_document(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_document_email(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_document_frame(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_document_presentation(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_document_spreadsheet(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_document_text(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_document_web(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_abstract(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_acknowledgments(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_afterword(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_appendix(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_backlink(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_biblioref(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_bibliography(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_chapter(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_colophon(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_conclusion(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_cover(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_credit(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_credits(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_dedication(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_endnote(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_endnotes(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_epigraph(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_epilogue(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_errata(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_example(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_footnote(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_foreword(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_glossary(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_glossref(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_index(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_introduction(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_noteref(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_pagelist(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_pagebreak(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_part(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_preface(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_prologue(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_pullquote(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_qna(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_subtitle(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_dpub_toc(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_drawing_area(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_editable_combo_box(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_editbar(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_embedded(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_entry(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_extended(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_feed(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_feed_article(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_figure(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_file_chooser(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_filler(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_focus_traversable(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_font_chooser(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_footer(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_footnote(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_form(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_frame(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_glass_pane(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_gui_list(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_grid(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_grid_cell(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_group(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_grouping(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_header(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_heading(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_html_container(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_horizontal_scrollbar(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_horizontal_separator(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_horizontal_slider(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_icon(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_icon_or_canvas(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_image(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_image_or_canvas(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_image_map(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_info_bar(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_inline_internal_frame(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_inline_list_item(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_inline_suggestion(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_input_method_window(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_internal_frame(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_invalid_role(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_label(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_label_or_caption(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_landmark(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_landmark_banner(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_landmark_complementary(
        obj: Atspi.Accessible,
        _role: Atspi.Role | None = None
    ) -> bool: ...

    @staticmethod
    def is_landmark_contentinfo(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_landmark_form(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_landmark_main(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_landmark_navigation(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_landmark_region(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_landmark_search(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_landmark_without_type(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_large_container(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_layered_pane(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_level_bar(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_link(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_list(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_list_box(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_list_box_item(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_list_item(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_log(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_live_region(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_mark(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_marquee(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_math(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_math_enclose(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_math_fenced(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_math_fraction(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_math_fraction_without_bar(
        obj: Atspi.Accessible, role: Atspi.Role | None = None
    ) -> bool: ...

    @staticmethod
    def is_math_layout_only(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_math_multi_script(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_math_related(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_math_root(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_math_square_root(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_math_sub_or_super_script(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_math_table(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_math_table_row(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_math_token(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_math_under_or_over_script(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_menu(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_menu_bar(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_menu_item(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_menu_item_of_any_kind(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_menu_related(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_modal_dialog(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_multi_line_entry(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_notification(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_option_pane(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_page(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_page_tab(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_page_tab_list(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_page_tab_list_related(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_panel(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_paragraph(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_password_text(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_popup_menu(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_progress_bar(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_push_button(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_push_button_menu(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_radio_button(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_radio_menu_item(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_rating(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_redundant_object_role(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_root_pane(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_row_header(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_ruler(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_scroll_bar(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_scroll_pane(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_section(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_separator(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_single_line_autocomplete_entry(
        obj: Atspi.Accessible, role: Atspi.Role | None = None
    ) -> bool: ...

    @staticmethod
    def is_single_line_entry(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_slider(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_spin_button(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_split_pane(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_static(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_status_bar(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_subscript(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_subscript_or_superscript(
        obj: Atspi.Accessible, role: Atspi.Role | None = None
    ) -> bool: ...

    @staticmethod
    def is_subscript_or_superscript_text(
        obj: Atspi.Accessible, role: Atspi.Role | None = None
    ) -> bool: ...

    @staticmethod
    def is_suggestion(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_superscript(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_svg(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_switch(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_table(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_table_cell(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_table_cell_or_header(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_table_column_header(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_table_header(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_table_related(
        obj: Atspi.Accessible,
        role: Atspi.Role | None = None,
        include_caption: bool = False
    ) -> bool: ...

    @staticmethod
    def is_table_row(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_table_row_header(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_tearoff_menu_item(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_terminal(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_text(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_text_input(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_text_input_date(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_text_input_email(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_text_input_number(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_text_input_search(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_text_input_telephone(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_text_input_time(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_text_input_url(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_text_input_week(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_time(obj: Atspi.Accessible, _role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_timer(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_title_bar(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_toggle_button(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_tool_bar(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_tool_tip(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_tree(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_tree_or_tree_table(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_tree_related(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_tree_item(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_tree_table(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_unknown(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_unknown_or_redundant(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_vertical_scrollbar(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_vertical_separator(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_vertical_slider(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_video(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_viewport(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_widget(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    @staticmethod
    def is_widget_controlled_by_line_navigation(
        obj: Atspi.Accessible, role: Atspi.Role | None = None
    ) -> bool: ...

    @staticmethod
    def is_web_element(obj: Atspi.Accessible, exclude_pseudo_elements: bool = True) -> bool: ...

    @staticmethod
    def is_web_element_custom(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_window(obj: Atspi.Accessible, role: Atspi.Role | None = None) -> bool: ...

    # From ax_utilities_state.py
    @staticmethod
    def get_current_item_status_string(obj: Atspi.Accessible) -> str: ...

    @staticmethod
    def has_no_state(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def has_popup(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def has_tooltip(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_active(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_animated(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_armed(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_busy(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_checkable(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_checked(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_collapsed(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_default(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_defunct(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_editable(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_enabled(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_expandable(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_expanded(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_focusable(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_focused(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_hidden(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_horizontal(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_iconified(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_indeterminate(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_invalid_state(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_invalid_entry(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_modal(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_multi_line(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_multiselectable(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_opaque(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_pressed(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_read_only(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_required(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_resizable(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_selectable(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_selectable_text(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_selected(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_sensitive(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_showing(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_single_line(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_stale(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_transient(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_truncated(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_vertical(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_visible(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def is_visited(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def manages_descendants(obj: Atspi.Accessible) -> bool: ...

    @staticmethod
    def supports_autocompletion(obj: Atspi.Accessible) -> bool: ...

    # From ax_utilities_collection.py
    @staticmethod
    def find_all_with_interfaces(
        root: Atspi.Accessible,
        interface_list: list[str],
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_with_role(
        root: Atspi.Accessible,
        roles_list: list[Atspi.Role],
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_without_roles(
        root: Atspi.Accessible,
        roles_list: list[Atspi.Role],
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_with_role_and_all_states(
        root: Atspi.Accessible,
        roles_list: list[Atspi.Role],
        state_list: list[Atspi.StateType],
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_with_role_and_any_state(
        root: Atspi.Accessible,
        roles_list: list[Atspi.Role],
        state_list: list[Atspi.StateType],
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_with_role_without_states(
        root: Atspi.Accessible,
        roles_list: list[Atspi.Role],
        state_list: list[Atspi.StateType],
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_with_states(
        root: Atspi.Accessible,
        state_list: list[Atspi.StateType],
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_with_any_state(
        root: Atspi.Accessible,
        state_list: list[Atspi.StateType],
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_without_states(
        root: Atspi.Accessible,
        state_list: list[Atspi.StateType],
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_accelerator_labels(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_alerts(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_animations(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_arrows(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_articles(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_audios(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_autocompletes(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_block_quotes(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_buttons(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_calendars(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_canvases(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_captions(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_charts(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_check_boxes(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_check_menu_items(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_clickables(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_color_choosers(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_column_headers(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_combo_boxes(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_comments(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_content_deletions(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_content_insertions(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_date_editors(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_definitions(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_description_lists(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_description_terms(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_description_values(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_desktop_frames(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_desktop_icons(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_dials(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_dialogs(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_dialogs_and_alerts(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_directory_panes(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_documents(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_document_emails(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_document_frames(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_document_presentations(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_document_spreadsheets(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_document_texts(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_document_webs(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_drawing_areas(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_editable_objects(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_editbars(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_embeddeds(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_entries(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_extendeds(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_file_choosers(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_fillers(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_focus_traversables(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_focusable_objects(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_focusable_objects_with_click_ancestor(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_focused_objects(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_font_choosers(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_footers(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_footnotes(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_forms(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_form_fields(
        root: Atspi.Accessible,
        must_be_focusable: bool = True,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_frames(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_glass_panes(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_grids(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_grid_cells(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_groupings(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_headers(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_headings(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_headings_at_level(
        root: Atspi.Accessible,
        level: int,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_html_containers(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_horizontal_scrollbars(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_horizontal_separators(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_horizontal_sliders(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_icons(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_icons_and_canvases(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_images(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_images_and_canvases(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_images_and_image_maps(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_image_maps(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_info_bars(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_input_method_windows(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_internal_frames(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_labels(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_labels_and_captions(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_landmarks(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_large_containers(
        root: Atspi.Accessible,
        pred: Callable[[Atspi.Accessible], bool] | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_layered_panes(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_level_bars(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_links(
        root: Atspi.Accessible,
        must_be_focusable: bool = True,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_lists(
        root: Atspi.Accessible,
        pred: Callable[[Atspi.Accessible], bool] | None = None,
        include_description_lists: bool = False,
        include_tab_lists: bool = False
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_list_boxes(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_list_items(
        root: Atspi.Accessible,
        pred: Callable[[Atspi.Accessible], bool] | None = None,
        include_description_terms: bool = False,
        include_tabs: bool = False
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_live_regions(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_logs(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_marks(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_marquees(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_maths(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_math_fractions(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_math_roots(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_menus(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_menu_bars(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_menu_items(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_menu_items_of_any_kind(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_menu_related_objects(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_modal_dialogs(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_multi_line_entries(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_notifications(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_option_panes(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_pages(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_page_tabs(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_page_tab_lists(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_page_tab_list_related_objects(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_panels(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_paragraphs(
        root: Atspi.Accessible,
        treat_headings_as_paragraphs: bool = False,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_password_texts(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_popup_menus(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_progress_bars(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_push_buttons(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_push_button_menus(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_radio_buttons(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_radio_menu_items(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_ratings(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_root_panes(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_row_headers(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_rulers(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_scroll_bars(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_scroll_panes(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_sections(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_selectable_objects(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_selected_objects(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_separators(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_set_containers(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_showing_objects(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_showing_and_visible_objects(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_showing_or_visible_objects(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_single_line_entries(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_sliders(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_spin_buttons(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_split_panes(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_statics(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_status_bars(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_subscripts(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_subscripts_and_superscripts(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_suggestions(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_superscripts(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_supports_action(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_supports_document(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_supports_editable_text(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_supports_hypertext(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_supports_hyperlink(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_supports_selection(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_supports_table(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_supports_table_cell(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_supports_text(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_supports_value(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_tables(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_table_cells(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_table_cells_and_headers(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_table_column_headers(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_table_headers(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_table_related_objects(
        root: Atspi.Accessible,
        pred: Callable | None = None,
        include_caption: bool = False
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_table_rows(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_table_row_headers(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_tearoff_menu_items(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_terminals(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_texts(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_text_inputs(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_timers(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_title_bars(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_toggle_buttons(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_tool_bars(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_tool_tips(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_trees(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_trees_and_tree_tables(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_tree_related_objects(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_tree_items(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_tree_tables(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_unrelated_labels(
        root: Atspi.Accessible,
        must_be_showing: bool = True,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_unvisited_links(
        root: Atspi.Accessible,
        must_be_focusable: bool = True,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_vertical_scrollbars(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_vertical_separators(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_vertical_sliders(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_videos(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_viewports(
        root: Atspi.Accessible, pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_visible_objects(
        root: Atspi.Accessible,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_all_visited_links(
        root: Atspi.Accessible,
        must_be_focusable: bool = True,
        pred: Callable | None = None
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def find_default_button(root: Atspi.Accessible) -> Atspi.Accessible | None: ...

    @staticmethod
    def find_focused_object(root: Atspi.Accessible) -> Atspi.Accessible | None: ...

    @staticmethod
    def find_info_bar(root: Atspi.Accessible) -> Atspi.Accessible | None: ...

    @staticmethod
    def find_status_bar(root: Atspi.Accessible) -> Atspi.Accessible | None: ...

    @staticmethod
    def has_combo_box_or_list_box(root: Atspi.Accessible) -> bool: ...

    @staticmethod
    def has_editable_object(root: Atspi.Accessible) -> bool: ...

    @staticmethod
    def has_scroll_pane(root: Atspi.Accessible) -> bool: ...

    @staticmethod
    def has_split_pane(root: Atspi.Accessible) -> bool: ...

    @staticmethod
    def has_tree_or_tree_table(root: Atspi.Accessible) -> bool: ...

    # From ax_utilities.relation.py
    @staticmethod
    def _get_relation_targets(
        obj: Atspi.Accessible,
        relation_type: Atspi.ReleationType
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_relation_targets_for_debugging(
        obj: Atspi.Accessible,
        relation_type: Atspi.ReleationType
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_controlled_by(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_controller_for(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_described_by(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_description_for(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_details(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_details_for(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_embedded_by(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_embeds(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_error_for(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_error_message(obj: Atspi.Accessible) -> str | None: ...

    @staticmethod
    def get_flows_from(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_flows_to(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_label_for(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_labelled_by(
        obj: Atspi.Accessible,
        exclude_ancestors: bool = True
    ) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_member_of(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_node_child_of(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_node_parent_of(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_parent_window_of(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_popup_for(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_subwindow_of(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def get_is_tooltip_for(obj: Atspi.Accessible) -> list[Atspi.Accessible]: ...

    @staticmethod
    def object_is_controlled_by(obj1: Atspi.Accessible, obj2: Atspi.Accessible) -> bool: ...

    @staticmethod
    def object_is_unrelated(obj: Atspi.Accessible) -> bool: ...



# Orca
#
# Copyright 2004-2009 Sun Microsystems Inc.
# Copyright 2010-2011 The Orca Team
# Copyright 2012 Igalia, S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=too-many-statements
# pylint: disable=wrong-import-position

"""The main module for the Orca screen reader."""

__id__        = "$Id$"
__version__   = "$Revision$"
__date__      = "$Date$"
__copyright__ = "Copyright (c) 2004-2009 Sun Microsystems Inc." \
                "Copyright (c) 2010-2011 The Orca Team" \
                "Copyright (c) 2012 Igalia, S.L."
__license__   = "LGPL"

import os
import signal
import sys

import gi
gi.require_version("Atspi", "2.0")
gi.require_version("Gdk", "3.0")
from gi.repository import Atspi
from gi.repository import Gdk
from gi.repository import Gio
from gi.repository import GLib

from . import braille
from . import braille_presenter
from . import clipboard
from . import dbus_service
from . import debug
from . import debugging_tools_manager
from . import event_manager
from . import focus_manager
from . import input_event_manager
from . import messages
from . import mouse_review
from . import orca_modifier_manager
from . import script_manager
from . import settings
from . import settings_manager
from . import speech_and_verbosity_manager
from . import sound
from . import systemd
from .ax_utilities import AXUtilities

def load_user_settings(script=None, skip_reload_message=False, is_reload=True):
    """(Re)Loads the user settings module, re-initializing things such as speech if necessary."""

    debug.print_message(debug.LEVEL_INFO, 'ORCA: Loading User Settings', True)

    if is_reload:
        sound.get_player().shutdown()
        speech_and_verbosity_manager.get_manager().shutdown_speech()
        braille.shutdown()
        mouse_review.get_reviewer().deactivate()

    event_manager.get_manager().pause_queuing(True, True, "Loading user settings.")
    if is_reload:
        _profile = settings_manager.get_manager().get_setting('activeProfile')[1]
        settings_manager.get_manager().set_profile(_profile)

    if script is None:
        script = script_manager.get_manager().get_default_script()

    settings_manager.get_manager().load_app_settings(script)
    if settings_manager.get_manager().get_setting('enableSpeech'):
        speech_and_verbosity_manager.get_manager().start_speech()
        if is_reload and not skip_reload_message:
            script.speak_message(messages.SETTINGS_RELOADED)

    if braille_presenter.get_presenter().get_braille_is_enabled():
        braille.init(input_event_manager.get_manager().process_braille_event)

    # TODO - JD: This ultimately belongs in an extension manager.
    mouse_reviewer = mouse_review.get_reviewer()
    if mouse_reviewer.get_is_enabled():
        mouse_reviewer.activate()

    if settings_manager.get_manager().get_setting('enableSound'):
        sound.get_player().init()

    # Handle the case where a change was made in the Orca Preferences dialog.
    orca_modifier_manager.get_manager().refresh_orca_modifiers("Loading user settings.")
    event_manager.get_manager().pause_queuing(False, False, "User settings loaded.")
    debug.print_message(debug.LEVEL_INFO, "ORCA: User Settings Loaded", True)

    manager = debugging_tools_manager.get_manager()
    manager.print_session_details()
    manager.print_running_applications(force=False)

def shutdown(script=None, _event=None, _signum=None):
    """Exits Orca. Returns True if shutdown ran to completion."""

    if hasattr(shutdown, "in_progress"):
        msg = "ORCA: Shutdown already in progress"
        debug.print_message(debug.LEVEL_INFO, msg, True)
        return False

    systemd.get_manager().notify_stopping()

    shutdown.in_progress = True

    def _timeout(_signum=None, _frame=None):
        msg = "TIMEOUT: something has hung. Aborting."
        debug.print_message(debug.LEVEL_SEVERE, msg, True)
        debugging_tools_manager.get_manager().print_running_applications(force=True)
        os.kill(os.getpid(), signal.SIGKILL)

    debug.print_message(debug.LEVEL_INFO, "ORCA: Shutting down", True)
    signal.signal(signal.SIGALRM, _timeout)
    signal.alarm(5)

    dbus_service.get_remote_controller().shutdown()

    orca_modifier_manager.get_manager().unset_orca_modifiers("Shutting down.")
    if script := script_manager.get_manager().get_active_script():
        if speech_and_verbosity_manager.get_manager().get_current_server():
            script.interrupt_presentation()
            script.present_message(messages.STOP_ORCA, reset_styles=False)

    # Pause event queuing first so that it clears its queue and will not accept new
    # events. Then let the script manager unregister script event listeners as well
    # as key grabs. Finally deactivate the event manager, which will also cause the
    # Atspi.Device to be set to None.
    event_manager.get_manager().pause_queuing(True, True, "Shutting down.")
    script_manager.get_manager().deactivate()
    event_manager.get_manager().deactivate()

    # TODO - JD: This ultimately belongs in an extension manager.
    clipboard.get_presenter().deactivate()
    mouse_review.get_reviewer().deactivate()

    # Shutdown all the other support.
    #
    if settings.enableSpeech:
        speech_and_verbosity_manager.get_manager().shutdown_speech()
    if braille_presenter.get_presenter().get_braille_is_enabled():
        braille.shutdown()
    if settings.enableSound:
        player = sound.get_player()
        player.shutdown()

    signal.alarm(0)
    debug.print_message(debug.LEVEL_INFO, 'ORCA: Quitting Atspi main event loop', True)
    Atspi.event_quit()
    debug.print_message(debug.LEVEL_INFO, 'ORCA: Shutdown complete', True)
    return True

def main():
    """The main entry point for Orca."""

    def _reload_on_signal(signum, frame):
        signal_string = f'({signal.strsignal(signum)})'
        tokens = [f"ORCA: Reloading due to signal={signum} {signal_string}", frame]
        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
        systemd.get_manager().notify_reloading()
        load_user_settings()
        systemd.get_manager().notify_ready()

    def _shutdown_on_signal(signum, frame):
        signal_string = f'({signal.strsignal(signum)})'
        tokens = [f"ORCA: Shutting down and exiting due to signal={signum} {signal_string}", frame]
        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
        shutdown()

    def _show_preferences_on_signal(signum, frame):
        signal_string = f'({signal.strsignal(signum)})'
        tokens = [f"ORCA: Showing preferences due to signal={signum} {signal_string}", frame]
        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
        manager = script_manager.get_manager()
        script = manager.get_active_script() or manager.get_default_script()
        if script:
            script.show_preferences_gui()

    signal.signal(signal.SIGHUP, _reload_on_signal)
    signal.signal(signal.SIGINT, _shutdown_on_signal)
    signal.signal(signal.SIGTERM, _shutdown_on_signal)
    signal.signal(signal.SIGQUIT, _shutdown_on_signal)
    signal.signal(signal.SIGUSR1, _show_preferences_on_signal)

    systemd.get_manager().start_watchdog()

    debug.print_message(debug.LEVEL_INFO, "ORCA: Enabling accessibility (if needed).", True)
    if not settings_manager.get_manager().is_accessibility_enabled():
        settings_manager.get_manager().set_accessibility(True)

    load_user_settings(is_reload=False)

    if not systemd.get_manager().is_systemd_managed():
        # Legacy behavior, here for backwards-compatibility. You really should
        # never rely on this. Run git blame and read the commit message!

        def _on_enabled_changed(gsetting, key):
            enabled = gsetting.get_boolean(key)
            msg = f"ORCA: {key} changed to {enabled}."
            debug.print_message(debug.LEVEL_INFO, msg, True)
            if key == "screen-reader-enabled" and not enabled:
                shutdown()

        try:
            _a11y_applications_gsetting = Gio.Settings(
                schema_id="org.gnome.desktop.a11y.applications")
            connection = _a11y_applications_gsetting.connect("changed", _on_enabled_changed)
            msg = f"ORCA: Connected to a11y applications gsetting: {bool(connection)}"
            debug.print_message(debug.LEVEL_INFO, msg, True)
        except GLib.Error as error:
            msg = (
                f"ORCA: EXCEPTION connecting to a11y applications (schema may be missing): {error}"
            )
            debug.print_message(debug.LEVEL_SEVERE, msg, True)
        except (AttributeError, TypeError) as error:
            msg = (
                "ORCA: EXCEPTION connecting to a11y applications (version incompatibility):"
                f"{error}"
            )
            debug.print_message(debug.LEVEL_SEVERE, msg, True)

    dbus_service.get_remote_controller().start()
    script = script_manager.get_manager().get_default_script()
    script.present_message(messages.START_ORCA)

    event_manager.get_manager().activate()
    script_manager.get_manager().activate()
    window = AXUtilities.find_active_window()
    if window and not focus_manager.get_manager().get_locus_of_focus():
        app = AXUtilities.get_application(window)
        focus_manager.get_manager().set_active_window(
            window, app, set_window_as_focus=True, notify_script=True)

        focused_object = focus_manager.get_manager().find_focused_object()
        if focused_object:
            focus_manager.get_manager().set_locus_of_focus(None, focused_object)

        script = script_manager.get_manager().get_script(app, focused_object or window)
        script_manager.get_manager().set_active_script(script, "Launching.")

    clipboard.get_presenter().activate()
    Gdk.notify_startup_complete()
    systemd.get_manager().notify_ready()

    try:
        debug.print_message(debug.LEVEL_INFO, "ORCA: Starting Atspi main event loop", True)
        Atspi.event_main()
    except GLib.Error as error:
        msg = f"ORCA: Exception starting ATSPI registry: {error}"
        debug.print_message(debug.LEVEL_SEVERE, msg, True)
        os.kill(os.getpid(), signal.SIGKILL)
    return 0

if __name__ == "__main__":
    sys.exit(main())



# Unit tests for focus_manager.py methods.
#
# Copyright 2025 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=wrong-import-position
# pylint: disable=import-outside-toplevel
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-statements
# pylint: disable=protected-access
# pylint: disable=too-many-arguments
# pylint: disable=too-many-positional-arguments
# pylint: disable=too-many-locals

"""Unit tests for focus_manager.py methods."""

from __future__ import annotations

from typing import TYPE_CHECKING
import gi
import pytest

gi.require_version("Atspi", "2.0")
from gi.repository import Atspi

if TYPE_CHECKING:
    from .orca_test_context import OrcaTestContext
    from unittest.mock import MagicMock

@pytest.mark.unit
class TestFocusManager:
    """Test FocusManager class methods."""

    def _setup_locus_of_focus_ax_mocks(
        self, test_context, mock_app=None, is_dead=False, cell_coords=(-1, -1), caret_offset=-1
    ):
        """Set up common AX object mocks for set_locus_of_focus testing scenarios.

        Returns:
            tuple: (mock_get_cell_coordinates, mock_get_caret_offset, mock_update_cached_text)
        """

        from orca.ax_object import AXObject
        from orca.ax_utilities import AXUtilities
        from orca.ax_table import AXTable
        from orca.ax_text import AXText

        test_context.patch_object(AXObject, "clear_cache", new=test_context.Mock())
        test_context.patch_object(
            AXObject, "is_dead", return_value=is_dead
        )
        test_context.patch_object(
            AXObject, "is_valid", side_effect=lambda obj: obj is not None
        )

        if mock_app is not None:
            test_context.patch_object(
                AXUtilities, "get_application", return_value=mock_app
            )
        else:
            test_context.patch_object(AXUtilities, "get_application", new=test_context.Mock())

        test_context.patch_object(
            AXUtilities, "save_object_info_for_events", new=test_context.Mock()
        )

        mock_get_cell_coordinates = test_context.Mock(return_value=cell_coords)
        test_context.patch_object(AXTable, "get_cell_coordinates", new=mock_get_cell_coordinates)

        mock_get_caret_offset = test_context.Mock(return_value=caret_offset)
        test_context.patch_object(AXText, "get_caret_offset", new=mock_get_caret_offset)

        mock_update_cached_text = test_context.Mock()
        test_context.patch_object(
            AXText, "update_cached_selected_text", new=mock_update_cached_text
        )

        return mock_get_cell_coordinates, mock_get_caret_offset, mock_update_cached_text

    def _setup_dependencies(self, test_context) -> dict[str, MagicMock]:
        """Set up mocks for focus_manager dependencies."""

        all_modules = [
            "orca.debug",
            "orca.messages",
            "orca.input_event",
            "orca.settings",
            "orca.keybindings",
            "orca.cmdnames",
            "orca.ax_object",
            "orca.settings_manager",
            "orca.dbus_service",
            "orca.script_manager",
            "orca.orca_i18n",
            "orca.guilabels",
            "orca.text_attribute_names",
            "orca.braille",
            "orca.ax_table",
            "orca.ax_text",
            "orca.ax_utilities",
        ]
        essential_modules = test_context._setup_essential_modules(all_modules)

        i18n_mock = essential_modules["orca.orca_i18n"]
        i18n_mock._ = lambda x: x  # Identity function for translations
        i18n_mock.C_ = lambda c, x: x  # Context-aware translations
        i18n_mock.ngettext = lambda s, p, n: s if n == 1 else p  # Plural forms

        debug_mock = essential_modules["orca.debug"]
        debug_mock.LEVEL_INFO = 800
        debug_mock.LEVEL_FINEST = 2
        debug_mock.LEVEL_SEVERE = 0
        debug_mock.print_message = test_context.Mock()
        debug_mock.println = test_context.Mock()

        keybindings_mock = essential_modules["orca.keybindings"]
        bindings_instance = test_context.Mock()
        bindings_instance.is_empty = test_context.Mock(return_value=True)
        bindings_instance.add = test_context.Mock()
        keybindings_mock.KeyBindings = test_context.Mock(return_value=bindings_instance)
        keybindings_mock.KeyBinding = test_context.Mock(
            return_value=test_context.Mock()
        )

        settings_manager_mock = essential_modules["orca.settings_manager"]
        manager_instance = test_context.Mock()
        manager_instance.get_setting = test_context.Mock(return_value=True)
        manager_instance.set_setting = test_context.Mock(return_value=True)
        settings_manager_mock.get_manager = test_context.Mock(return_value=manager_instance)

        braille_mock = essential_modules["orca.braille"]
        braille_mock.setBrlapiPriority = test_context.Mock()
        braille_mock.BRLAPI_PRIORITY_HIGH = 1

        script_manager_mock = essential_modules["orca.script_manager"]
        script_mgr_instance = test_context.Mock()
        script_instance = test_context.Mock()
        script_instance.locus_of_focus_changed = test_context.Mock()
        script_mgr_instance.get_active_script = test_context.Mock(
            return_value=script_instance
        )
        script_mgr_instance.get_script = test_context.Mock(return_value=script_instance)
        script_mgr_instance.set_active_script = test_context.Mock()
        script_manager_mock.get_manager = test_context.Mock(return_value=script_mgr_instance)

        ax_object_mock = essential_modules["orca.ax_object"]
        ax_object_mock.AXObject = test_context.Mock()
        ax_object_mock.AXObject.is_dead = test_context.Mock(return_value=False)
        ax_object_mock.AXObject.is_valid = test_context.Mock(return_value=True)
        ax_object_mock.AXObject.clear_cache = test_context.Mock()
        ax_object_mock.AXObject.is_ancestor = test_context.Mock(return_value=False)
        ax_object_mock.AXObject.has_broken_ancestry = test_context.Mock(return_value=False)

        ax_table_mock = essential_modules["orca.ax_table"]
        ax_table_mock.AXTable = test_context.Mock()
        ax_table_mock.AXTable.get_cell_coordinates = test_context.Mock(return_value=(-1, -1))

        ax_text_mock = essential_modules["orca.ax_text"]
        ax_text_mock.AXText = test_context.Mock()
        ax_text_mock.AXText.get_caret_offset = test_context.Mock(return_value=-1)
        ax_text_mock.AXText.update_cached_selected_text = test_context.Mock()

        ax_utilities_mock = essential_modules["orca.ax_utilities"]
        ax_utilities_mock.AXUtilities = test_context.Mock()
        ax_utilities_mock.AXUtilities.is_table_cell = test_context.Mock(return_value=False)
        ax_utilities_mock.AXUtilities.get_application = test_context.Mock()
        ax_utilities_mock.AXUtilities.save_object_info_for_events = test_context.Mock()
        ax_utilities_mock.AXUtilities.is_active = test_context.Mock(return_value=True)
        ax_utilities_mock.AXUtilities.get_focused_object = test_context.Mock()

        return essential_modules

    def test_init(self, test_context) -> None:
        """Test FocusManager.__init__."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()
        assert manager._focus is None
        assert manager._window is None
        assert manager._object_of_interest is None
        assert manager._active_mode is None

    def test_clear_state(self, test_context: OrcaTestContext) -> None:
        """Test FocusManager.clear_state."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()
        test_context.patch_object(
            manager, "_focus", new=test_context.Mock(spec=Atspi.Accessible)
        )
        test_context.patch_object(
            manager, "_window", new=test_context.Mock(spec=Atspi.Accessible)
        )
        test_context.patch_object(
            manager, "_object_of_interest", new=test_context.Mock(spec=Atspi.Accessible)
        )
        manager._active_mode = "focus-tracking"

        manager.clear_state()
        assert manager._focus is None
        assert manager._window is None
        assert manager._object_of_interest is None
        assert manager._active_mode is None
        test_context.patch_object(
            manager, "_focus", new=test_context.Mock(spec=Atspi.Accessible)
        )
        manager.clear_state("test reason")
        assert manager._focus is None

    def test_find_focused_object(self, test_context: OrcaTestContext) -> None:
        """Test FocusManager.find_focused_object."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager
        from orca.ax_utilities import AXUtilities

        manager = FocusManager()
        mock_window = test_context.Mock(spec=Atspi.Accessible)
        mock_focused = test_context.Mock(spec=Atspi.Accessible)
        manager._window = mock_window
        mock_get_focused = test_context.Mock(return_value=mock_focused)
        test_context.patch_object(AXUtilities, "get_focused_object", new=mock_get_focused)
        result = manager.find_focused_object()
        assert result == mock_focused
        mock_get_focused.assert_called_once_with(mock_window)

    @pytest.mark.parametrize(
        "focus, window, expected",
        [
            pytest.param(None, None, True, id="both_none"),
            pytest.param(None, "window", False, id="focus_none_window_set"),
            pytest.param("focus", None, False, id="focus_set_window_none"),
            pytest.param("focus", "window", False, id="both_set"),
        ],
    )
    def test_focus_and_window_are_unknown(self, test_context, focus, window, expected) -> None:
        """Test FocusManager.focus_and_window_are_unknown."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()
        focus_obj = test_context.Mock(spec=Atspi.Accessible) if focus else None
        window_obj = test_context.Mock(spec=Atspi.Accessible) if window else None
        test_context.patch_object(manager, "_focus", new=focus_obj)
        test_context.patch_object(manager, "_window", new=window_obj)
        result = manager.focus_and_window_are_unknown()
        assert result == expected

    @pytest.mark.parametrize(
        "focus_obj,focus_is_dead,expected",
        [
            pytest.param("mock_focus", True, True, id="focus_is_dead"),
            pytest.param("mock_focus", False, False, id="focus_is_alive"),
            pytest.param(None, False, False, id="no_focus_set"),
        ],
    )
    def test_focus_is_dead(self, test_context, focus_obj, focus_is_dead, expected) -> None:
        """Test FocusManager.focus_is_dead with various focus states."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()

        if focus_obj == "mock_focus":
            mock_focus = test_context.Mock(spec=Atspi.Accessible)
            manager._focus = mock_focus
            expected_call_arg = mock_focus
        else:
            manager._focus = None
            expected_call_arg = None

        from orca.ax_object import AXObject

        mock_is_dead = test_context.Mock(return_value=focus_is_dead)
        test_context.patch_object(AXObject, "is_dead", new=mock_is_dead)

        result = manager.focus_is_dead()
        assert result == expected
        mock_is_dead.assert_called_once_with(expected_call_arg)

    @pytest.mark.parametrize(
        "focus, window, expected",
        [
            pytest.param(None, "window", False, id="no_focus"),
            pytest.param("same_obj", "same_obj", True, id="focus_equals_window"),
            pytest.param("focus_obj", "window_obj", False, id="focus_not_window"),
        ],
    )
    def test_focus_is_active_window(self, test_context, focus, window, expected) -> None:
        """Test FocusManager.focus_is_active_window."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()
        if focus == "same_obj" and window == "same_obj":
            same_obj = test_context.Mock(spec=Atspi.Accessible)
            manager._focus = same_obj
            manager._window = same_obj
        else:
            focus_obj = test_context.Mock(spec=Atspi.Accessible) if focus else None
            window_obj = test_context.Mock(spec=Atspi.Accessible) if window else None
            test_context.patch_object(manager, "_focus", new=focus_obj)
            test_context.patch_object(manager, "_window", new=window_obj)
        result = manager.focus_is_active_window()
        assert result == expected

    @pytest.mark.parametrize(
        "focus, window, is_ancestor, expected",
        [
            pytest.param(None, "window", False, False, id="no_focus"),
            pytest.param("focus", "window", True, True, id="focus_in_window"),
            pytest.param("focus", "window", False, False, id="focus_not_in_window"),
        ],
    )
    def test_focus_is_in_active_window(
        self, test_context, focus, window, is_ancestor, expected
    ) -> None:
        """Test FocusManager.focus_is_in_active_window."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()
        focus_obj = test_context.Mock(spec=Atspi.Accessible) if focus else None
        window_obj = test_context.Mock(spec=Atspi.Accessible) if window else None
        test_context.patch_object(manager, "_focus", new=focus_obj)
        test_context.patch_object(manager, "_window", new=window_obj)
        from orca.ax_object import AXObject

        mock_is_ancestor = test_context.Mock(return_value=is_ancestor)
        test_context.patch_object(AXObject, "is_ancestor", new=mock_is_ancestor)
        result = manager.focus_is_in_active_window()
        assert result == expected
        if focus and window:
            mock_is_ancestor.assert_called_once_with(manager._focus, manager._window)

    @pytest.mark.parametrize(
        "start_offset, end_offset, mode, use_null_object",
        [
            pytest.param(None, None, None, False, id="all_defaults"),
            pytest.param(5, None, None, False, id="start_only"),
            pytest.param(5, 10, None, False, id="start_and_end"),
            pytest.param(5, 10, "caret-tracking", False, id="all_specified"),
            pytest.param(None, None, None, True, id="null_object"),
        ],
    )
    def test_emit_region_changed_scenarios(  # pylint: disable=too-many-arguments
        self, test_context, start_offset, end_offset, mode, use_null_object
    ) -> None:
        """Test FocusManager.emit_region_changed with various scenarios."""
        essential_modules = self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager, FOCUS_TRACKING, FLAT_REVIEW

        manager = FocusManager()

        if use_null_object:
            manager.emit_region_changed(None)
            assert manager._object_of_interest is None
        else:
            mock_obj = test_context.Mock(spec=Atspi.Accessible)
            test_context.patch_object(mock_obj, "emit", new=test_context.Mock())
            expected_start = start_offset if start_offset is not None else 0
            expected_end = end_offset if end_offset is not None else expected_start
            expected_mode = mode if mode is not None else FOCUS_TRACKING

            manager.emit_region_changed(mock_obj, start_offset, end_offset, mode)
            mock_obj.emit.assert_any_call("mode-changed::" + expected_mode, 1, "")
            mock_obj.emit.assert_any_call("region-changed", expected_start, expected_end)
            assert manager._active_mode == expected_mode
            assert manager._object_of_interest == mock_obj

            manager.emit_region_changed(mock_obj, mode=FLAT_REVIEW)
            braille_mock = essential_modules["orca.braille"]
            braille_mock.setBrlapiPriority.assert_called_with(braille_mock.BRLAPI_PRIORITY_HIGH)

            manager.emit_region_changed(mock_obj, mode=FOCUS_TRACKING)
            braille_mock.setBrlapiPriority.assert_called_with()

    @pytest.mark.parametrize(
        "active_mode, expected",
        [
            pytest.param("say-all", True, id="in_say_all"),
            pytest.param("focus-tracking", False, id="not_in_say_all"),
            pytest.param(None, False, id="no_mode"),
        ],
    )
    def test_in_say_all(self, test_context, active_mode, expected) -> None:
        """Test FocusManager.in_say_all."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()
        manager._active_mode = active_mode
        result = manager.in_say_all()
        assert result == expected

    def test_get_active_mode_and_object_of_interest(self, test_context: OrcaTestContext) -> None:
        """Test FocusManager.get_active_mode_and_object_of_interest."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()
        mock_obj = test_context.Mock(spec=Atspi.Accessible)
        mode = "caret-tracking"
        manager._active_mode = mode
        manager._object_of_interest = mock_obj
        result_mode, result_obj = manager.get_active_mode_and_object_of_interest()
        assert result_mode == mode
        assert result_obj == mock_obj

    def test_get_locus_of_focus(self, test_context: OrcaTestContext) -> None:
        """Test FocusManager.get_locus_of_focus."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()
        mock_focus = test_context.Mock(spec=Atspi.Accessible)
        manager._focus = mock_focus
        result = manager.get_locus_of_focus()
        assert result == mock_focus

    @pytest.mark.parametrize(
        "force, obj_equals_current_focus, notify_script",
        [
            pytest.param(True, True, True, id="force_same_object"),
            pytest.param(False, True, True, id="no_force_same_object"),
            pytest.param(False, False, True, id="different_object"),
            pytest.param(False, False, False, id="different_object_no_notify"),
        ],
    )
    def test_set_locus_of_focus(
        self, test_context, force, obj_equals_current_focus, notify_script
    ) -> None:
        """Test FocusManager.set_locus_of_focus."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()
        mock_event = test_context.Mock(spec=Atspi.Event)
        mock_obj = test_context.Mock(spec=Atspi.Accessible)
        mock_app = test_context.Mock(spec=Atspi.Accessible)
        if obj_equals_current_focus:
            manager._focus = mock_obj
        else:
            test_context.patch_object(
                manager, "_focus", new=test_context.Mock(spec=Atspi.Accessible)
            )
        script_manager_mock = essential_modules["orca.script_manager"]
        manager_instance = script_manager_mock.get_manager.return_value
        script_instance = manager_instance.get_active_script.return_value
        self._setup_locus_of_focus_ax_mocks(test_context, mock_app=mock_app)
        mock_emit = test_context.Mock()
        test_context.patch_object(manager, "emit_region_changed", new=mock_emit)
        manager.set_locus_of_focus(mock_event, mock_obj, notify_script, force)
        if not force and obj_equals_current_focus:
            mock_emit.assert_not_called()
        else:
            assert manager._focus == mock_obj
            if notify_script:
                mock_emit.assert_called_once()
                script_instance.locus_of_focus_changed.assert_called_once()
            else:
                mock_emit.assert_not_called()

    def test_set_locus_of_focus_null_object(self, test_context: OrcaTestContext) -> None:
        """Test FocusManager.set_locus_of_focus with null object."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager
        from orca.ax_object import AXObject

        test_context.patch_object(
            AXObject, "is_valid", side_effect=lambda obj: obj is not None
        )
        manager = FocusManager()
        mock_focus = test_context.Mock(spec=Atspi.Accessible)
        manager._focus = mock_focus
        manager.set_locus_of_focus(None, None)
        assert manager._focus is None

    def test_set_locus_of_focus_dead_object(self, test_context: OrcaTestContext) -> None:
        """Test FocusManager.set_locus_of_focus with dead object."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        self._setup_locus_of_focus_ax_mocks(test_context, is_dead=True)
        manager = FocusManager()
        mock_obj = test_context.Mock(spec=Atspi.Accessible)
        old_focus = test_context.Mock(spec=Atspi.Accessible)
        manager._focus = old_focus
        manager.set_locus_of_focus(None, mock_obj)

        # Focus should not change since object is dead
        assert manager._focus == old_focus

    def test_active_window_is_active(self, test_context: OrcaTestContext) -> None:
        """Test FocusManager.active_window_is_active."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()
        mock_window = test_context.Mock(spec=Atspi.Accessible)
        manager._window = mock_window
        from orca.ax_object import AXObject
        from orca.ax_utilities import AXUtilities

        mock_clear = test_context.Mock()
        test_context.patch_object(AXObject, "clear_cache", new=mock_clear)
        mock_is_active = test_context.Mock(return_value=True)
        test_context.patch_object(AXUtilities, "is_active", new=mock_is_active)
        result = manager.active_window_is_active()
        assert result is True
        mock_clear.assert_called_once_with(
            mock_window, False, "Ensuring the active window is really active."
        )
        mock_is_active.assert_called_once_with(mock_window)

    def test_get_active_window(self, test_context: OrcaTestContext) -> None:
        """Test FocusManager.get_active_window."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()
        mock_window = test_context.Mock(spec=Atspi.Accessible)
        manager._window = mock_window
        result = manager.get_active_window()
        assert result == mock_window

    @pytest.mark.parametrize(
        "frame_equals_current, set_window_as_focus, focus_is_in_window, has_broken_ancestry",
        [
            pytest.param(True, False, True, False, id="same_window"),
            pytest.param(False, True, False, False, id="set_as_focus"),
            pytest.param(False, False, False, False, id="focus_not_in_window"),
            pytest.param(False, False, False, True, id="focus_has_broken_ancestry"),
        ],
    )
    def test_set_active_window(
        self,
        test_context,
        frame_equals_current,
        set_window_as_focus,
        focus_is_in_window,
        has_broken_ancestry,
    ) -> None:
        """Test FocusManager.set_active_window."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()
        mock_frame = test_context.Mock(spec=Atspi.Accessible)
        mock_app = test_context.Mock(spec=Atspi.Accessible)
        mock_focus = test_context.Mock(spec=Atspi.Accessible)
        if frame_equals_current:
            manager._window = mock_frame
        else:
            test_context.patch_object(
                manager, "_window", new=test_context.Mock(spec=Atspi.Accessible)
            )
        manager._focus = mock_focus
        from orca.ax_object import AXObject
        from orca.ax_utilities import AXUtilities

        mock_set_focus = test_context.Mock()
        test_context.patch_object(manager, "set_locus_of_focus", new=mock_set_focus)
        test_context.patch_object(
            manager, "focus_is_active_window", return_value=focus_is_in_window
        )
        test_context.patch_object(
            manager, "focus_is_in_active_window", return_value=focus_is_in_window
        )
        test_context.patch_object(
            AXObject, "has_broken_ancestry", return_value=has_broken_ancestry
        )
        test_context.patch_object(
            AXUtilities, "get_application", return_value=mock_app
        )
        manager.set_active_window(mock_frame, mock_app, set_window_as_focus, False)
        if not frame_equals_current:
            assert manager._window == mock_frame
        if set_window_as_focus:
            mock_set_focus.assert_called_with(None, mock_frame, False)
        elif not focus_is_in_window and not has_broken_ancestry:
            mock_set_focus.assert_called_with(None, mock_frame, notify_script=True)

    def test_set_active_window_null_frame(self, test_context: OrcaTestContext) -> None:
        """Test FocusManager.set_active_window with null frame."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        manager = FocusManager()
        mock_focus = test_context.Mock(spec=Atspi.Accessible)
        manager._focus = mock_focus
        manager.set_active_window(None)
        assert manager._window is None

    def test_get_manager(self, test_context: OrcaTestContext) -> None:
        """Test focus_manager.get_manager."""

        self._setup_dependencies(test_context)
        from orca import focus_manager

        manager1 = focus_manager.get_manager()
        manager2 = focus_manager.get_manager()
        assert manager1 is manager2
        assert isinstance(manager1, focus_manager.FocusManager)

    def test_set_locus_of_focus_table_cell_coordinates(self, test_context: OrcaTestContext) -> None:
        """Test FocusManager.set_locus_of_focus saves table cell coordinates."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        mock_get_cell_coordinates = self._setup_locus_of_focus_ax_mocks(
            test_context, cell_coords=(5, 3)
        )[0]
        manager = FocusManager()
        mock_obj = test_context.Mock(spec=Atspi.Accessible)
        manager.set_locus_of_focus(None, mock_obj)

        mock_get_cell_coordinates.assert_called_once_with(mock_obj, find_cell=True)
        assert manager.get_last_cell_coordinates() == (5, 3)

    def test_set_locus_of_focus_text_cursor_position(self, test_context: OrcaTestContext) -> None:
        """Test FocusManager.set_locus_of_focus saves text cursor position."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        mocks = self._setup_locus_of_focus_ax_mocks(test_context, caret_offset=42)
        mock_get_caret_offset = mocks[1]
        mock_update_cached_text = mocks[2]
        manager = FocusManager()
        mock_obj = test_context.Mock(spec=Atspi.Accessible)
        manager.set_locus_of_focus(None, mock_obj)

        mock_get_caret_offset.assert_called_once_with(mock_obj)
        mock_update_cached_text.assert_called_once_with(mock_obj)
        obj, offset = manager.get_last_cursor_position()
        assert obj == mock_obj
        assert offset == 42

    def test_set_locus_of_focus_none_object_coordinates(
        self, test_context: OrcaTestContext
    ) -> None:
        """Test FocusManager.set_locus_of_focus handles None object coordinates properly."""

        self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager
        from orca.ax_table import AXTable
        from orca.ax_text import AXText
        from orca.ax_object import AXObject

        test_context.patch_object(
            AXObject, "is_valid", side_effect=lambda obj: obj is not None
        )
        manager = FocusManager()

        mock_get_cell_coordinates = test_context.Mock()
        test_context.patch_object(AXTable, "get_cell_coordinates", new=mock_get_cell_coordinates)
        mock_get_caret_offset = test_context.Mock()
        test_context.patch_object(AXText, "get_caret_offset", new=mock_get_caret_offset)
        mock_update_cached_selected_text = test_context.Mock()
        test_context.patch_object(
            AXText, "update_cached_selected_text", new=mock_update_cached_selected_text
        )
        manager.set_locus_of_focus(None, None)

        mock_get_cell_coordinates.assert_not_called()
        mock_get_caret_offset.assert_not_called()
        mock_update_cached_selected_text.assert_not_called()

        assert manager.get_last_cell_coordinates() == (-1, -1)

    @pytest.mark.parametrize(
        "method_name,attribute_name,test_offset",
        [
            pytest.param(
                "get_penultimate_cursor_position",
                "_penultimate_cursor_position",
                42,
                id="penultimate",
            ),
            pytest.param("get_last_cursor_position", "_last_cursor_position", 123, id="last"),
        ],
    )
    def test_cursor_position_debug_logging(
        self, test_context: OrcaTestContext, method_name, attribute_name, test_offset
    ) -> None:
        """Test cursor position methods log debug information."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        focus_manager = FocusManager()
        mock_obj = test_context.Mock(spec=Atspi.Accessible)
        setattr(focus_manager, attribute_name, (mock_obj, test_offset))

        method = getattr(focus_manager, method_name)
        obj, offset = method()

        assert obj is mock_obj
        assert offset == test_offset
        essential_modules["orca.debug"].print_tokens.assert_called()

    def test_set_locus_of_focus_null_object_with_debug(self, test_context: OrcaTestContext) -> None:
        """Test FocusManager.set_locus_of_focus with null object logs debug message."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        focus_manager = FocusManager()
        initial_focus = test_context.Mock(spec=Atspi.Accessible)
        focus_manager._focus = initial_focus
        focus_manager.set_locus_of_focus(None, None)

        assert focus_manager._focus is None
        essential_modules["orca.debug"].print_message.assert_called()

    def test_set_locus_of_focus_with_event_and_force_flag(
        self, test_context: OrcaTestContext
    ) -> None:
        """Test FocusManager.set_locus_of_focus with event and force flag."""

        essential_modules = self._setup_dependencies(test_context)
        from orca.focus_manager import FocusManager

        focus_manager = FocusManager()
        mock_obj = test_context.Mock(spec=Atspi.Accessible)
        mock_event = test_context.Mock()
        focus_manager.set_locus_of_focus(mock_event, mock_obj, force=True)
        essential_modules["orca.debug"].print_tokens.assert_called()
